"""Script for running tsunami inundation scenario for Karratha, WA, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in project.outputtimedir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a simulated submarine landslide.

Ole Nielsen and Duncan Gray, GA - 2005 and Jane Sexton, Nick Bartzis, GA - 2006
"""

# FIXME - parallel library not yet ready (16/10/2006)



#------------------------------------------------------------------------------
# Import necessary modules
#------------------------------------------------------------------------------

# Standard modules
from os import sep
from os.path import dirname, basename
from os import mkdir, access, F_OK
from shutil import copy
import time
import sys


# Related major packages
from anuga.shallow_water import Domain
from anuga.shallow_water import Dirichlet_boundary
from anuga.shallow_water import File_boundary
from anuga.shallow_water import Reflective_boundary

from anuga.pmesh.mesh_interface import create_mesh_from_regions

from anuga.geospatial_data.geospatial_data import *

# Application specific imports
import project                 # Definition of file names and polygons

from anuga_parallel.parallel_api import distribute, myid, numprocs

#------------------------------------------------------------------------------
# Copy scripts to time stamped output directory and capture screen
# output to file
#------------------------------------------------------------------------------

# filenames
meshname = project.meshname+'.msh'
source_dir = project.boundarydir

if myid == 0:
    # creates copy of code in output dir if dir doesn't exist
    if access(project.outputtimedir,F_OK) == 0:
        mkdir (project.outputtimedir)
    copy (dirname(project.__file__) +sep+ project.__name__+'.py',
          project.outputtimedir + project.__name__+'.py')
    copy (__file__, project.outputtimedir + basename(__file__))
    print 'project.outputtimedir',project.outputtimedir


    #--------------------------------------------------------------------------
    # Create the triangular mesh based on overall clipping polygon with a
    # tagged
    # boundary and interior regions defined in project.py along with
    # resolutions (maximal area of per triangle) for each polygon
    #--------------------------------------------------------------------------


    print 'start create mesh from regions'
    from caching import cache
    meshname = project.meshname + '_%d.msh' %myid
    _ = cache(create_mesh_from_regions,
              project.bounding_polygon,
              {'boundary_tags': {'back': [7, 8], 'side': [0, 6],
                                 'ocean': [1, 2, 3, 4, 5]},
               'maximum_triangle_area': 200000,
               'filename': meshname},
              verbose = True,
              evaluate = False)

    #-------------------------------------------------------------------------
    # Setup computational domain
    #-------------------------------------------------------------------------
    domain = Domain(meshname, use_cache = True, verbose = True)
    print domain.statistics()
    domain.set_name(project.basename)
    domain.set_datadir(project.outputtimedir)


    #-------------------------------------------------------------------------
    # Setup initial conditions
    #-------------------------------------------------------------------------
    tide = 0.
    domain.set_quantity('stage', tide)
    domain.set_quantity('friction', 0.0) 
    domain.set_quantity('elevation', 
                        filename = project.datadir + project.basename + '.pts',
                        use_cache = False,
                        verbose = True,
                        alpha = 0.1)
    
    #-------------------------------------------------------------------------
    # Setup boundary conditions 
    #-------------------------------------------------------------------------

    print 'Available boundary tags', domain.get_boundary_tags()
    #Bf = File_boundary(source_dir + project.boundary_basename + '.sww', 
    #                   domain, verbose = True)
    Br = Reflective_boundary(domain)
    Bd = Dirichlet_boundary([tide,0,0])
    domain.set_boundary({'back': Br,
                         'side': Bd,
                         'ocean': None}) # Bind this one later
else:
    domain = None


#--------------------------------------------------------------------------
# Create the parallel domain
#--------------------------------------------------------------------------
if numprocs > 1:
    domain = distribute(domain, verbose=True)

# Set those boundaries that can't be communicated automatically
# FIXME: There is a problem with the parallel domain.
# It seems that boundaries are outside coverage.
#
# We get
#
#------------------------------------------------
#Interpolation_function (spatio-temporal) statistics:
#  Extent:
#    x in [-60494.651593, 149593.652382], len(x) == 56
#    y in [-74198.017534, 184623.446077], len(y) == 56
#    t in [0.000000, 28610.000000], len(t) == 145
#  Quantities:
#    stage in [-1.797799, 1.644574]
#    xmomentum in [-33.219473, 21.571094]
#    ymomentum in [-32.775090, 73.761256]
#  Interpolation points (xi, eta): number of points == 411
#    xi in [-420467.297166, -368252.119852]
#    eta in [-7652664.149808, -7605199.504709]
#  Interpolated quantities (over all timesteps):
#    stage at interpolation points in [inf, inf]
#    xmomentum at interpolation points in [inf, inf]
#    ymomentum at interpolation points in [inf, inf]
#------------------------------------------------
#
#Instead of
#
#------------------------------------------------
#Interpolation_function (spatio-temporal) statistics:
#  Extent:
#    x in [-60494.651593, 149593.652382], len(x) == 56
#    y in [-74198.017534, 184623.446077], len(y) == 56
#    t in [0.000000, 28610.000000], len(t) == 145
#  Quantities:
#    stage in [-1.797799, 1.644574]
#    xmomentum in [-33.219473, 21.571094]
#    ymomentum in [-32.775090, 73.761256]
#  Interpolation points (xi, eta): number of points == 503
#    xi in [4747.280215, 96909.729282]
#    eta in [9240.613414, 83183.465791]
#  Interpolated quantities (over all timesteps):
#    stage at interpolation points in [-0.909515, 1.331521]
#    xmomentum at interpolation points in [-17.054914, 17.110528]
#    ymomentum at interpolation points in [-24.691402, 16.625063]
#------------------------------------------------


Bf = File_boundary(source_dir + project.boundary_basename + '.sww',
                   domain, verbose = True)
domain.set_boundary({'ocean': Bf})


#----------------------------------------------------------------------------
# Evolve system through time
#----------------------------------------------------------------------------
import time
t0 = time.time()

for t in domain.evolve(yieldstep = 60, finaltime = 28600):
    domain.write_time()
    domain.write_boundary_statistics(tags = 'ocean')     
    
print 'That took %.2f seconds' %(time.time()-t0)

